/**************************************************************************\
* Targa averager v1.1 by Warp                                              *
* ---------------------------                                              *
*                                                                          *
* Reads a series of uncompressed 24-bit true color targa files and         *
* calculates their average, addition or difference                         *
*                                                                          *
* See average.txt for more info.                                           *
*                                                                          *
\**************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>

/*****************/
/* The file info */
/*****************/
typedef struct FileInfo
{ char* Name;
  char* DestName;
  char* Operand;
  char** InputFiles;
  int Index,Len;
  long Min,Max,DestMin;
  int DestIndex,DestLen;
  unsigned int Mode;
  int Wid,Heig;
} FileInfo;
/* Mode:
   -Bits 0-3:
     0 = Average
     1 = Add
     2 = Difference
   -Bits 4-5:
     0 = Source is multiple (name must have "[begin-end]")
     1 = Source is a single file
     2 = Source is a file listing file
   -Bit 6:
     0 = Only two parameters
     1 = Three parameters (source file(s), operand file, destination file(s))
   -Bit 7:
     0 = Destination is a single file
     1 = Destination is multiple (name must have "[<begin>]")
*/
#define OpType(n) ((n)&15)
#define SrcType(n) (((n)>>4)&3)
#define ParamType(n) (((n)>>6)&1)
#define DestType(n) (((n)>>7)&1)

#define LineMaxLength 500
char* LineBuffer=0;

/***********************/
/* Check if empty line */
/***********************/
int EmptyLine(char* line)
{ int i=0;
  while(line[i]) if(line[i++]>32) return 0;
  return 1;
}

/***********************/
/* Count lines in file */
/***********************/
long CountLines(FILE* file)
{ long Pos=ftell(file);
  long Cnt=0;

  while(fgets(LineBuffer,LineMaxLength,file)!=NULL)
    if(!EmptyLine(LineBuffer)) Cnt++;

  fseek(file,Pos,SEEK_SET);
  return Cnt;
}

/**************************************************************/
/* Scan a string for the first occurence of a given character */
/* and return index to it. -1 if c does not occur in s        */
/**************************************************************/
int StrChr(char* s,char c)
{ int i;
  for(i=0;s[i];i++) if(s[i]==c) return i;
  return -1;
}

/***************************************************************/
/* Compare two strings, case insensitive. 1=equal, 0=different */
/***************************************************************/
int StrCmp(char* s1,char* s2)
{ int i=0;
  do
  { if(tolower(s1[i])!=tolower(s2[i])) return 0;
    i++;
  } while(s1[i-1] && s2[i-1]);
  return 1;
}

/******************************************/
/* Read file names from file listing file */
/******************************************/
int ReadFilenames(FILE* InFile,char** Dest)
{ long i,ind=0;

  while(fgets(LineBuffer,LineMaxLength,InFile)!=NULL)
  { if(!EmptyLine(LineBuffer))
    { i=StrChr(LineBuffer,'\n'); if(i>-1) LineBuffer[i]=0;
      Dest[ind]=(char*)malloc(strlen(LineBuffer)+1);
      if(Dest[ind]==NULL) return 1;
      strcpy(Dest[ind],LineBuffer);
      ind++;
    }
  }

  return 0;
}

/************************/
/* Fill FileInfo struct */
/************************/
int GetFileInfo(char* params[],int n,FileInfo* I)
{ int ind1,ind2,mid,i;
  char* f;
  long Cnt;
  FILE* InputFiles;

/*-----------------------------------*
 |  Check source file(s)             |
 *-----------------------------------*/
  f=params[0];
  if((ind1=StrChr(f,'['))==-1 ||
     (ind2=StrChr(f,']'))==-1 ||
     (mid=StrChr(f,'-'))==-1)
  { /* if source is a single file: */
    if(StrCmp(&f[strlen(f)-4],".tga"))
    { I->Mode|=1<<4;
      I->Name=f;
      I->Min=I->Max=1;
    }
    /* if source is a file listing file: */
    else
    { if((InputFiles=fopen(f,"r"))==NULL)
      { fprintf(stderr,"Can't open "); perror(f);
        return 1;
      }
      I->Mode|=2<<4;
      LineBuffer=(char*)malloc(LineMaxLength);
      if(LineBuffer==NULL)
      { fprintf(stderr,"Not enough memory.\n"); return 1;
      }

      Cnt=CountLines(InputFiles);
      if(Cnt==0)
      { fprintf(stderr,"Input file list is empty.\n"); return 1;
      }
      I->InputFiles=(char**)malloc(Cnt*sizeof(char*));
      if(I->InputFiles==NULL || ReadFilenames(InputFiles,I->InputFiles))
      { fprintf(stderr,"Not enough memory.\n"); return 1;
      }

      I->Min=1; I->Max=Cnt;
      fclose(InputFiles);
    }
  }
  /* if source is multiple: */
  else
  { if(mid<ind1+2 || ind2<mid+2 || mid-ind1!=ind2-mid)
    { fprintf(stderr,"<begin> and <end> must be equal in length.\n");
      return 1;
    }

    I->Index=ind1;
    I->Len=mid-ind1-1;

    for(i=0;i<I->Len;i++)
    { if(!isdigit(f[i+ind1+1]) || !isdigit(f[i+mid+1]))
      { fprintf(stderr,"<begin> and <end> must be digits.\n");
        return 1;
      }
    }

    I->Min=atol(&f[ind1+1]);
    I->Max=atol(&f[mid+1]);

    if(I->Min>I->Max)
    { fprintf(stderr,"<begin> must be less or equal to <end>\n");
      return 1;
    }

    I->Name=(char*)malloc(strlen(f)-(ind2-ind1+1)+I->Len+1);
    if(I->Name==NULL)
    { fprintf(stderr,"Not enough memory.\n"); return 1;
    }
    for(i=0;i<ind1;i++) I->Name[i]=f[i];
    for(i=ind1;i<ind1+I->Len;i++) I->Name[i]=' ';
    for(i=ind1+I->Len;f[++ind2];i++) I->Name[i]=f[ind2];
    I->Name[i]=0;
  }

/*-----------------------------------*
 |  Check operand file               |
 *-----------------------------------*/
  if(n==3)
  { I->Mode|=1<<6;
    I->Operand=params[1];
  }

/*-----------------------------------*
 |  Check destination file(s)        |
 *-----------------------------------*/
  f=params[n-1];
  if((ind1=StrChr(f,'['))==-1)
  { /* if destination is a single file: */
    I->DestName=f;
  }
  else
  { /* if destination is multiple: */
    I->Mode|=1<<7;
    ind2=StrChr(f,']');
    if(ind2==-1 || ind1>ind2 || ind2==ind1+1)
    { fprintf(stderr,
              "Multiple destination file name must have '[<begin>]'\n");
      return 1;
    }
    for(i=ind1+1;i<ind2;i++)
    { if(!isdigit(f[i]))
      { fprintf(stderr,"<begin> in destination filename must be digit.\n");
        return 1;
      }
    }

    I->DestMin=atol(&f[ind1+1]);
    I->DestIndex=ind1;

    if(SrcType(I->Mode)==1) I->DestLen=1;
    else
    { Cnt=I->DestMin+(I->Max-I->Min+1);
      I->DestLen=0;
      while(Cnt) { I->DestLen++; Cnt/=10; }
    }

    I->DestName=(char*)malloc(strlen(f)-(ind2-ind1+1)+I->DestLen+1);
    if(I->DestName==NULL)
    { fprintf(stderr,"Not enough memory.\n"); return 1;
    }
    for(i=0;i<ind1;i++) I->DestName[i]=f[i];
    for(i=ind1;i<ind1+I->DestLen;i++) I->DestName[i]=' ';
    for(i=ind1+I->DestLen;f[++ind2];i++) I->DestName[i]=f[ind2];
    I->DestName[i]=0;
  }

/*---------------------------------------------*
 |  Check if parameter combination makes sense |
 *---------------------------------------------*/
  if(SrcType(I->Mode)==1 && ParamType(I->Mode)==0)
  { fprintf(stderr,
      "When you specify a single source file, you need to specify also an\n"
      "operand file and a destination file.\n");
    return 1;
  }
  if(SrcType(I->Mode)==1 && DestType(I->Mode)==1)
  { fprintf(stderr,
      "A multiple destination makes no sense with a single source file.\n");
    return 1;
  }
  if(SrcType(I->Mode)!=1 && ParamType(I->Mode)==1 && DestType(I->Mode)==0)
  { fprintf(stderr,
      "When you specify multiple source files and an operand file, the\n"
      "destination must be multiple too.\n");
    return 1;
  }
  if(SrcType(I->Mode)!=1 && ParamType(I->Mode)==0 && DestType(I->Mode)==1 &&
     I->Min==I->Max)
  { fprintf(stderr,
      "There must be at least 2 source files when averaging from multiple\n"
      "files to multiple files.\n");
    return 1;
  }

  return 0;
}

/****************/
/* Print syntax */
/****************/
void PrintSyntax(void)
{ puts(
    "Targa averager v1.1 by Warp.\n\n"
    "Usage: average [-(a|d)] <inputfile(s)> [<operand file>] "
      "<outputfile(s)>\n\n"
    "  Options:\n"
    "    -a : Calculate the addition of the images\n"
    "    -d : Calculate the difference of the images (see average.txt)\n"
    "    Default is averaging.\n\n"
    "  <inputfile(s)> can be:\n"
    "  - A single file (extension must be .tga)\n"
    "  - A file listing filename (extension other than .tga) "
      "(see average.txt)\n"
    "  - A multiple file name:\n"
    "    Use [<begin>-<end>] in <inputfiles>, for example:\n"
    "    average pic[0-5].tga result.tga (averages pic0.tga to pic5.tga)\n"
    "    average pic[01-24].tga result.tga (averages pic01.tga to "
      "pic24.tga)\n"
    "    average pic[001-100].tga result.tga (averages pic001.tga to "
      "pic100.tga)\n\n"
    "  <outputfile(s)> can be:\n"
    "  - A single file\n"
    "  - A multiple file name:\n"
    "    Use [<begin>] where the numbers will be located, for example:\n"
    "    average -d pic[01-20].tga diff[1].tga");
}

/************************************/
/* Selects the nth source file name */
/************************************/
void SelectSrcFile(FileInfo* I,long n)
{ long i,num;

  if(SrcType(I->Mode)==1) return;

  if(SrcType(I->Mode)==0)
  { num=n+I->Min-1;
    for(i=I->Index+I->Len-1;i>=I->Index;i--)
    { I->Name[i]=num%10+'0'; num/=10;
    }
  }
  else if(SrcType(I->Mode)==2)
  { I->Name=I->InputFiles[n-1];
  }
}

/*****************************************/
/* Selects the nth destination file name */
/*****************************************/
void SelectDestFile(FileInfo* I,long n)
{ int i;

  if(DestType(I->Mode)==0) return;

  n+=I->DestMin-1;
  for(i=I->DestIndex+I->DestLen-1;i>=I->DestIndex;i--)
  { I->DestName[i]=n%10+'0'; n/=10;
  }
}

/****************************************/
/* Create a targa file (outputs header) */
/****************************************/
FILE* CreateTarga(char* FileName,char* Header,char* ImageID)
{ static int overwrite=0;
  FILE* OutFile;
  char c;

  if(overwrite==0 && (OutFile=fopen(FileName,"r"))!=NULL)
  { fclose(OutFile);
    printf("\nFile '%s' exists. Overwrite? (y/n/a) ",FileName);
    do
    { c=getc(stdin);
    } while(c!='y' && c!='n' && c!='a');
    if(c=='a') overwrite=1;
    else if(c!='y') return NULL;
  }
  if((OutFile=fopen(FileName,"wb"))==NULL)
  { fprintf(stderr,"Can't create "); perror(FileName);
    return NULL;
  }
  fwrite(Header,1,18,OutFile);
  if(Header[0]) fwrite(ImageID,1,Header[0],OutFile);

  return OutFile;
}

/*************************************************************/
/* Calculate addition or difference between two char buffers */
/*************************************************************/
void AddChar2Char(unsigned char* Dest,unsigned char* Src,long Len,int which)
{ long i;
  switch(which)
  { case 1: /* Addition */
      for(i=0;i<Len;i++) Dest[i]+=Src[i];
      break;
    case 2: /* Difference */
      for(i=0;i<Len;i++) Dest[i]-=Src[i];
      break;
  }
}

/********************************************************/
/* Add char buffer to long int buffer */
/********************************************************/
void AddChar2Long(unsigned long* Dest,unsigned char* Src,long Len)
{ long i;
  for(i=0;i<Len;i++) Dest[i]+=Src[i];
}

/***********************\
|* Averaging functions *|
|* ------------------- *|
\***********************/

/*****************************************/
/* Average multiple files to single file */
/*****************************************/
void MultipleSingle(FileInfo* I,unsigned long* Ave,unsigned char* Buf,
                    unsigned char *Header,char* ImageID,int SlowMethod)
{ FILE *InFile,*OutFile;
  long i,y,amount=I->Max-I->Min+1,Ptr=18+Header[0],BufLen=I->Wid*I->Heig*3;
  unsigned char* Buf2=(unsigned char*)Ave;

  if((OutFile=CreateTarga(I->DestName,Header,ImageID))==NULL)
    return;

  printf("0%%"); fflush(stdout);
  if(SlowMethod) /* Slow method */
  { for(y=0;y<I->Heig;y++)
    { if(OpType(I->Mode)==0) for(i=0;i<I->Wid*3;i++) Ave[i]=0;
      else for(i=0;i<I->Wid*3;i++) Buf2[i]=0;

      for(i=1;i<=amount;i++)
      { SelectSrcFile(I,i);
        if((InFile=fopen(I->Name,"rb"))==NULL)
        { fprintf(stderr,"Can't open "); perror(I->Name); return;
        }
        fseek(InFile,Ptr,SEEK_SET);
        fread(Buf,3,I->Wid,InFile);
        fclose(InFile);

        if(OpType(I->Mode)==0)
          AddChar2Long(Ave,Buf,I->Wid*3);
        else
          AddChar2Char(Buf2,Buf,I->Wid*3,OpType(I->Mode));
      }

      if(OpType(I->Mode)==0)
      { for(i=0;i<I->Wid*3;i++) Buf[i]=Ave[i]/amount;
        fwrite(Buf,3,I->Wid,OutFile);
      }
      else
        fwrite(Buf2,3,I->Wid,OutFile);

      Ptr+=I->Wid*3;
      printf("\r%i%%",(y+1)*100/I->Heig); fflush(stdout);
    }
  }
  else /* Fast method */
  { if(OpType(I->Mode)==0) for(i=0;i<BufLen;i++) Ave[i]=0;
    else for(i=0;i<BufLen;i++) Buf2[i]=0;

    for(i=1;i<=amount;i++)
    { SelectSrcFile(I,i);
      if((InFile=fopen(I->Name,"rb"))==NULL)
      { fprintf(stderr,"Can't open "); perror(I->Name); return;
      }
      fseek(InFile,Ptr,SEEK_SET);
      fread(Buf,1,BufLen,InFile);
      fclose(InFile);

      if(OpType(I->Mode)==0)
        AddChar2Long(Ave,Buf,BufLen);
      else
        AddChar2Char(Buf2,Buf,BufLen,OpType(I->Mode));

      printf("\r%i%%",i*100/amount); fflush(stdout);
    }
    puts("");
    if(OpType(I->Mode)==0)
    { for(i=0;i<BufLen;i++) Buf[i]=Ave[i]/amount;
      fwrite(Buf,1,BufLen,OutFile);
    }
    else
      fwrite(Buf2,1,BufLen,OutFile);
  }

  fclose(OutFile);
}

/********************************************/
/* Average multiple files to multiple files */
/********************************************/
void MultipleMultiple(FileInfo* I,unsigned long* Ave,unsigned char* Buf,
                      unsigned char *Header,char* ImageID,int SlowMethod)
{ FILE *InFile1,*InFile2,*OutFile;
  long i,j,y,amount=I->Max-I->Min+1,Ptr=18+Header[0],BufLen=I->Wid*I->Heig*3;
  unsigned char* Buf2=(unsigned char*)Ave,*tmp;

  printf("0%%"); fflush(stdout);
  if(SlowMethod) /* Slow method */
  { for(y=0;y<I->Heig;y++)
    { SelectSrcFile(I,1);
      if((InFile2=fopen(I->Name,"rb"))==NULL)
      { fprintf(stderr,"Can't open "); perror(I->Name); return;
      }
      fseek(InFile2,Ptr,SEEK_SET);
      if(OpType(I->Mode)>0)
      { fread(Buf,3,I->Wid,InFile2);
        if(OpType(I->Mode)==1) { tmp=Buf; Buf=Buf2; Buf2=tmp; }
      }

      for(i=2;i<=amount;i++)
      { InFile1=InFile2;
        if(OpType(I->Mode)==0)
        { fseek(InFile1,Ptr,SEEK_SET);
          fread(Buf,3,I->Wid,InFile1);
          for(j=0;j<I->Wid*3;j++) Ave[j]=Buf[j];
        }
        else if(OpType(I->Mode)==2) { tmp=Buf; Buf=Buf2; Buf2=tmp; }

        SelectSrcFile(I,i);
        if((InFile2=fopen(I->Name,"rb"))==NULL)
        { fprintf(stderr,"Can't open "); perror(I->Name); return;
        }
        fseek(InFile2,Ptr,SEEK_SET);
        fread(Buf,3,I->Wid,InFile2);

        SelectDestFile(I,i-1);
        if(y==0)
        { if((OutFile=CreateTarga(I->DestName,Header,ImageID))==NULL)
            return;
        }
        else
        { if((OutFile=fopen(I->DestName,"r+b"))==NULL)
          { fprintf(stderr,"Can't open "); perror(I->DestName); return;
          }
          fseek(OutFile,Ptr,SEEK_SET);
        }

        if(OpType(I->Mode)==0)
        { AddChar2Long(Ave,Buf,I->Wid*3);
          for(j=0;j<I->Wid*3;j++) Buf[j]=Ave[j]>>1;
          fwrite(Buf,3,I->Wid,OutFile);
        }
        else
        { if(OpType(I->Mode)==2)
            for(j=0;j<I->Wid*3;j++) Buf2[j]=Buf[j]-Buf2[j];
          else AddChar2Char(Buf2,Buf,I->Wid*3,OpType(I->Mode));
          fwrite(Buf2,3,I->Wid,OutFile);
        }

        fclose(InFile1); fclose(OutFile);
      }
      fclose(InFile2);

      Ptr+=I->Wid*3;
      printf("\r%i%%",(y+1)*100/I->Heig); fflush(stdout);
    }
  }
  else /* Fast method */
  { SelectSrcFile(I,1);
    if((InFile2=fopen(I->Name,"rb"))==NULL)
    { fprintf(stderr,"Can't open "); perror(I->Name); return;
    }
    fseek(InFile2,Ptr,SEEK_SET);
    if(OpType(I->Mode)>0)
    { fread(Buf,1,BufLen,InFile2);
      if(OpType(I->Mode)==1) { tmp=Buf; Buf=Buf2; Buf2=tmp; }
    }

    for(i=2;i<=amount;i++)
    { InFile1=InFile2;
      if(OpType(I->Mode)==0)
      { fseek(InFile1,Ptr,SEEK_SET);
        fread(Buf,1,BufLen,InFile1);
        for(j=0;j<BufLen;j++) Ave[j]=Buf[j];
      }
      else if(OpType(I->Mode)==2) { tmp=Buf; Buf=Buf2; Buf2=tmp; }

      SelectSrcFile(I,i);
      if((InFile2=fopen(I->Name,"rb"))==NULL)
      { fprintf(stderr,"Can't open "); perror(I->Name); return;
      }
      fseek(InFile2,Ptr,SEEK_SET);
      fread(Buf,1,BufLen,InFile2);

      SelectDestFile(I,i-1);
      if((OutFile=CreateTarga(I->DestName,Header,ImageID))==NULL)
        return;

      if(OpType(I->Mode)==0)
      { AddChar2Long(Ave,Buf,BufLen);
        for(j=0;j<BufLen;j++) Buf[j]=Ave[j]>>1;
        fwrite(Buf,1,BufLen,OutFile);
      }
      else
      { if(OpType(I->Mode)==2)
          for(j=0;j<BufLen;j++) Buf2[j]=Buf[j]-Buf2[j];
        else AddChar2Char(Buf2,Buf,BufLen,OpType(I->Mode));
        fwrite(Buf2,1,BufLen,OutFile);
      }

      fclose(InFile1); fclose(OutFile);
      printf("\r%i%%",i*100/amount); fflush(stdout);
    }
    fclose(InFile2);
  }
}

/**************************************************************/
/* Average multiple files with operand file to multiple files */
/**************************************************************/
void MultipleOperandMultiple(FileInfo* I,unsigned long* Ave,
                             unsigned char* Buf,unsigned char *Header,
                             char* ImageID,int SlowMethod)
{ FILE *InFile,*Operand,*OutFile;
  long i,j,y,amount=I->Max-I->Min+1,Ptr=18+Header[0],BufLen=I->Wid*I->Heig*3;
  unsigned char* Buf2=(unsigned char*)Ave,*tmp;

  if((Operand=fopen(I->Operand,"rb"))==NULL)
  { fprintf(stderr,"Can't open "); perror(I->Operand); return;
  }

  printf("0%%"); fflush(stdout);
  if(SlowMethod) /* Slow method */
  { for(i=1;i<=amount;i++)
    { SelectSrcFile(I,i);
      if((InFile=fopen(I->Name,"rb"))==NULL)
      { fprintf(stderr,"Can't open "); perror(I->Name); return;
      }

      SelectDestFile(I,i);
      if((OutFile=CreateTarga(I->DestName,Header,ImageID))==NULL)
        return;

      fseek(InFile,Ptr,SEEK_SET);
      fseek(Operand,Ptr,SEEK_SET);

      for(y=0;y<I->Heig;y++)
      { fread(Buf,3,I->Wid,Operand);
        if(OpType(I->Mode)==0) for(j=0;j<I->Wid*3;j++) Ave[j]=Buf[j];
        else { tmp=Buf; Buf=Buf2; Buf2=tmp; }

        fread(Buf,3,I->Wid,InFile);

        if(OpType(I->Mode)==0)
        { AddChar2Long(Ave,Buf,I->Wid*3);
          for(j=0;j<I->Wid*3;j++) Buf[j]=Ave[j]>>1;
        }
        else
          AddChar2Char(Buf,Buf2,I->Wid*3,OpType(I->Mode));

        fwrite(Buf,3,I->Wid,OutFile);
      }
      fclose(InFile); fclose(OutFile);

      printf("\r%i%%",i*100/amount); fflush(stdout);
    }
  }
  else /* Fast method */
  { if(OpType(I->Mode)>0)
    { fseek(Operand,Ptr,SEEK_SET);
      fread(Buf2,1,BufLen,Operand);
    }

    for(i=1;i<=amount;i++)
    { if(OpType(I->Mode)==0)
      { fseek(Operand,Ptr,SEEK_SET);
        fread(Buf,1,BufLen,Operand);
        for(j=0;j<BufLen;j++) Ave[j]=Buf[j];
      }

      SelectSrcFile(I,i);
      if((InFile=fopen(I->Name,"rb"))==NULL)
      { fprintf(stderr,"Can't open "); perror(I->Name); return;
      }
      fseek(InFile,Ptr,SEEK_SET);
      fread(Buf,1,BufLen,InFile);
      fclose(InFile);

      if(OpType(I->Mode)==0)
      { AddChar2Long(Ave,Buf,BufLen);
        for(j=0;j<BufLen;j++) Buf[j]=Ave[j]>>1;
      }
      else
        AddChar2Char(Buf,Buf2,BufLen,OpType(I->Mode));

      SelectDestFile(I,i);
      if((OutFile=CreateTarga(I->DestName,Header,ImageID))==NULL)
        return;
      fwrite(Buf,1,BufLen,OutFile);
      fclose(OutFile);

      printf("\r%i%%",i*100/amount); fflush(stdout);
    }
  }

  fclose(Operand);
}

/***************************************************************************/
/* Main routine                                                            */
/* ------------                                                            */
/***************************************************************************/
int main(int argc,char* argv[])
{ FileInfo Info;
  int ArgPtr=1,SlowMethod=0;
  FILE* InFile;
  unsigned char Header[18];
  unsigned char* Buf;
  unsigned long* Ave;
  char* ImageID=NULL;

/* Check options */
/* ------------- */
  if(argc>ArgPtr && argv[ArgPtr][0]=='-')
  { if(argv[ArgPtr][1]=='a') Info.Mode=1;
    else if(argv[ArgPtr][1]=='d') Info.Mode=2;
    else
    { printf("Invalid option: %s\n",argv[ArgPtr]);
      return 1;
    }
    ArgPtr++;
  }
  else Info.Mode=0;

/* Syntax */
/* ------ */
  if(argc<ArgPtr+2 || argc>ArgPtr+3)
  { PrintSyntax();
    return 0;
  }

/* Fill FileInfo struct */
/* -------------------- */
  if(GetFileInfo(&argv[ArgPtr],argc-ArgPtr,&Info)) return 1;

/* Read and check tga header */
/* ------------------------- */
  SelectSrcFile(&Info,1);
  if((InFile=fopen(Info.Name,"rb"))==NULL)
  { fprintf(stderr,"Can't open "); perror(Info.Name);
    return 1;
  }
  fread(Header,1,18,InFile);
  if(Header[2]!=2 || Header[16]!=24)
  { fprintf(stderr,
      "Only uncompressed 24-bit true color TGA files supported.\n");
    return 1;
  }
  Info.Wid=Header[12]+Header[13]*256;
  Info.Heig=Header[14]+Header[15]*256;

  if(Header[0]) /* Check for Image ID */
  { ImageID=(char*)malloc(Header[0]);
    if(ImageID==NULL) { fprintf(stderr,"Not enough memory.\n"); return 1; }
    fread(ImageID,1,Header[0],InFile);
  }

  fclose(InFile);

/* Allocate some buffers */
/* --------------------- */
  /* First try fast method */
  if(OpType(Info.Mode)==0)
    Ave=(unsigned long*)malloc(Info.Wid*Info.Heig*3*sizeof(unsigned long));
  else
    Ave=(unsigned long*)malloc(Info.Wid*Info.Heig*3);
  Buf=(unsigned char*)malloc(Info.Wid*Info.Heig*3);

  if(Ave==NULL || Buf==NULL)
  { /* If not enough memory, try slower method */
    if(Ave) free(Ave);
    if(Buf) free(Buf);
    SlowMethod=1;
    if(OpType(Info.Mode)==0)
      Ave=(unsigned long*)malloc(Info.Wid*3*sizeof(unsigned long));
    else
      Ave=(unsigned long*)malloc(Info.Wid*3);
    Buf=(unsigned char*)malloc(Info.Wid*3);
    if(Ave==NULL || Buf==NULL)
    { fprintf(stderr,"Not enough memory.\n"); return 1;
    }
    puts("Not enough memory for fast averaging. Using slow method.");
  }

/* Average the images */
/* ------------------ */

  if(SrcType(Info.Mode)==1)
    MultipleOperandMultiple(&Info,Ave,Buf,Header,ImageID,SlowMethod);
  else
  { if(ParamType(Info.Mode)==0)
    { if(DestType(Info.Mode)==0)
        MultipleSingle(&Info,Ave,Buf,Header,ImageID,SlowMethod);
      else
        MultipleMultiple(&Info,Ave,Buf,Header,ImageID,SlowMethod);
    }
    else
    { MultipleOperandMultiple(&Info,Ave,Buf,Header,ImageID,SlowMethod);
    }
  }

/* Clean up */
  free(Buf); free(Ave);
  if(LineBuffer) free(LineBuffer);
  return 0;
}